# -*- coding: utf-8 -*-
from requests_oauthlib import OAuth1Session

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME

from ..utils import reverse_with_host

class SSOClient(OAuth1Session):

    request_token_url = '%(HOST)s/%(REQUEST_TOKEN_PATH)s' % settings.PASSAPORTE_WEB
    access_token_url = '%(HOST)s/%(ACCESS_TOKEN_PATH)s' % settings.PASSAPORTE_WEB
    authorization_url = '%(HOST)s/%(AUTHORIZATION_PATH)s' % settings.PASSAPORTE_WEB
    user_data_url = '%(HOST)s/%(FETCH_USER_DATA_PATH)s' % settings.PASSAPORTE_WEB

    def __init__(self, **kwargs):
        client_key = settings.PASSAPORTE_WEB['CONSUMER_TOKEN']
        client_secret = settings.PASSAPORTE_WEB['CONSUMER_SECRET']
        callback_uri = reverse_with_host('sso_consumer:callback')

        return super(SSOClient, self).__init__(
            client_key, client_secret=client_secret, callback_uri=callback_uri, **kwargs
        )

    def fetch_request_token(self, realm=None):
        return super(SSOClient, self).fetch_request_token(self.request_token_url, realm=realm)

    def authorize(self, request):
        request_token = self.fetch_request_token()

        request.session['request_token'] = {request_token.get('oauth_token'): request_token.get('oauth_token_secret')}
        request.session[REDIRECT_FIELD_NAME] = request.GET.get(REDIRECT_FIELD_NAME, settings.LOGIN_REDIRECT_URL)
        request.session.save()

        return '{0}?oauth_token={1}'.format(
            self.authorization_url, request_token.get('oauth_token')
        )

    def fetch_access_token(self):
        return super(SSOClient, self).fetch_access_token(self.access_token_url)



# -*- coding: utf-8 -*-
import os
from vcr import VCR
import requests

from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib.auth import authenticate, login
from django.http import HttpRequest
from django.test.client import Client
from django.test import TestCase
from django.utils.importlib import import_module

from passaporte_web.tests.helpers import TEST_USER

from identity_client import PERSISTENCE_MODULE


__all__ = [
    'MyfcIDTestClient', 'MyfcIDTestCase', 'MyfcIDAPITestCase', 'use_cassette', 'use_sso_cassette'
]


def use_cassette(*args, **kwargs):
    return VCR(
        cassette_library_dir = os.path.join(os.path.dirname(__file__), 'cassettes', 'api_client'),
        match_on = ['url', 'method', 'headers', 'body'],
        record_mode = 'none',
    ).use_cassette(*args, **kwargs)

def use_sso_cassette(*args, **kwargs):
    return VCR(
        cassette_library_dir = os.path.join(os.path.dirname(__file__), 'cassettes', 'sso'),
        match_on = ['method', 'scheme', 'host', 'port', 'path', 'query'],
        record_mode = 'none',
    ).use_cassette(*args, **kwargs)

def full_oauth_dance(client, url=reverse('sso_consumer:request_token')):
    with use_sso_cassette('fetch_user_data/active_request_token'):
        initiate_response = client.get(url)

        authorization_url = initiate_response['Location']
        authentication_challenge = requests.get(authorization_url)

        authorization_data = {
            'email': TEST_USER['email'],
            'password': TEST_USER['password'],
            'csrfmiddlewaretoken': u'3SB7jwwt8ALf2bl6DBjF27TdS2oAinlb',
            'next': authorization_url,
        }
        authentication_response = requests.post(
            authentication_challenge.url, authorization_data,
            headers = {'Referer': authorization_url},
            cookies = authentication_challenge.cookies,
        )

        callback_url = u'http://testserver/sso/callback/?oauth_token=JL0KLSpLpmWHOMwf&oauth_verifier=28583669'
        return client.get(callback_url)


class MyfcIDTestClient(Client):

    def login(self, **credentials):
        """
        Sets the Client to appear as if it has successfully logged into a site.

        Returns True if login is possible; False if the provided credentials
        are incorrect, or the user is inactive, or if the sessions framework is
        not available.
        """
        user = authenticate(**credentials)
        if user:
            engine = import_module(settings.SESSION_ENGINE)

            # Create a fake request to store login details.
            request = HttpRequest()
            if self.session:
                request.session = self.session
            else:
                request.session = engine.SessionStore()
            login(request, user)

            # Save the session values.
            request.session.save()

            # Set the cookie to represent the session.
            session_cookie = settings.SESSION_COOKIE_NAME
            self.cookies[session_cookie] = request.session.session_key
            cookie_data = {
                'max-age': None,
                'path': '/',
                'domain': settings.SESSION_COOKIE_DOMAIN,
                'secure': settings.SESSION_COOKIE_SECURE or None,
                'expires': None,
            }
            self.cookies[session_cookie].update(cookie_data)

            return True
        else:
            return False


class MyfcIDTestCase(PERSISTENCE_MODULE.TestCase):

    userdata = {
        'login' : 'cesar',
        'fullname' : 'Cesar Lustosa',
        'email' : 'cesar@lustosa.net',
        'activation_key': 'Ad6e1161446611529d976dcfb9027eb55',
        'id' : 275310,
        'password': 'e10adc3949ba59abbe56e057f20f883e',
    }

    def __call__(self, result=None):
        """
        Wrapper around default __call__ method to perform common Django test
        set up. This means that user-defined Test Cases aren't required to
        include a call to super().setUp().
        """
        self.client = MyfcIDTestClient()
        try:
            self._pre_setup()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            import sys
            result.addError(self, sys.exc_info())
            return
        super(PERSISTENCE_MODULE.TestCase, self).__call__(result)
        try:
            self._post_teardown()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception:
            import sys
            result.addError(self, sys.exc_info())
            return


class MyfcIDAPITestCase(MyfcIDTestCase):

    http_method = None
    url_name = None
    url = None

    def make_request(self, url, *args, **kwargs):
        client_method = getattr(self.client, self.http_method.lower())
        return client_method(url, *args, **kwargs)


    def test_unauthorized_service(self):
        self.service.acl = []
        self.service.save()

        response = self.make_request(self.url, {},
            HTTP_ACCEPT = 'application/json',
            HTTP_AUTHORIZATION = self.auth,
        )

        self.assertEqual(response.status_code, 403)


    def test_unauthenticated_service(self):

        response = self.make_request(self.url, {},
            HTTP_ACCEPT = 'application/json',
        )

        self.assertEqual(response.status_code, 403)



# -*- coding: utf-8 -*-
import logging
from mock import Mock, patch

from django.utils.importlib import import_module
from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings

import identity_client

from identity_client.models import Identity
from identity_client.utils import get_account_module
from identity_client.tests.test_sso_client import SSOClientRequestToken
from identity_client.tests.test_decorators import (
    OAuthCallbackWithoutRequestToken,
    OAuthCallbackWithRequestToken,
    SIDE_EFFECTS,
)
from .helpers import full_oauth_dance
from .mock_helpers import patch_request

__all__ = ['InitiateSSO', 'FetchUserData']


class InitiateSSO(TestCase):

    def test_initiate_redirects_user_to_authorization_url_on_success(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 302)
        authorization_url = '%(HOST)s/%(AUTHORIZATION_PATH)s' % settings.PASSAPORTE_WEB
        self.assertEqual(
            response['Location'],
            '{0}?oauth_token={1}'.format(authorization_url, SSOClientRequestToken.REQUEST_TOKEN['oauth_token'])
        )

        return

    @patch('logging.error', Mock())
    def test_initiate_adds_oauth_data_to_session_on_success(self):
        expected_token = SSOClientRequestToken.REQUEST_TOKEN['oauth_token']
        expected_token_secret = SSOClientRequestToken.REQUEST_TOKEN['oauth_token_secret']

        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        session = self.client.session
        self.assertTrue('request_token' in session)
        self.assertTrue(expected_token in session['request_token'])
        self.assertEqual(session['request_token'][expected_token], expected_token_secret)

        return

    @patch('logging.error', Mock())
    def test_initiate_fails_if_service_credentials_are_invalid(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/invalid_credentials'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 503)
        self.assertEqual(
            logging.error.call_args[0][0],
            u"Error invoking initiate: Token request failed with code 401, response was 'Unauthorized consumer with key 'not a valid token''."
        )

    @patch('logging.error', Mock())
    def test_initiate_fails_if_connection_to_provider_fails(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/500'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 503)
        self.assertEqual(
            logging.error.call_args[0][0],
            u"Error invoking initiate: Token request failed with code 500, response was 'Internal Server Error'."
        )

    @patch('logging.error', Mock())
    def test_initiate_fails_if_connection_to_provider_times_out(self):
        with patch_request(Mock(side_effect=SIDE_EFFECTS['Timeout'])):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 503)
        self.assertEqual(logging.error.call_count, 1)
        self.assertEqual(
            logging.error.call_args[0][0],
            u'Error invoking initiate: connection: Read timed out. (read timeout=30)'
        )

    @patch('logging.error', Mock())
    def test_initiate_fails_if_provider_is_not_available(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/503'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 503)
        self.assertEqual(logging.error.call_count, 1)
        self.assertEqual(
            logging.error.call_args[0][0],
            u"Error invoking initiate: Token request failed with code 503, response was 'Service Unavailable'."
        )

    @patch('logging.error', Mock())
    def test_initiate_fails_if_request_has_an_error(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/418'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 503)
        self.assertEqual(logging.error.call_count, 1)
        self.assertEqual(
            logging.error.call_args[0][0],
            u"Error invoking initiate: Token request failed with code 418, response was 'I'm a Teapot'."
        )

    @patch('logging.error', Mock())
    def test_initiate_fails_gracefully_when_response_is_malformed(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/malformed_response'):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 500)
        self.assertEqual(logging.error.call_count, 1)
        self.assertEqual(
            logging.error.call_args[0][0],
            'Error invoking initiate: Unable to decode token from token response'
        )

    @patch('logging.error', Mock())
    def test_initiate_fails_gracefully(self):
        with patch_request(Mock(side_effect=SIDE_EFFECTS['Memory'])):
            response = self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertEqual(response.status_code, 500)
        self.assertEqual(logging.error.call_count, 1)
        self.assertEqual(
            logging.error.call_args[0][0] % logging.error.call_args[0][1:],
            u"Error invoking initiate: KTHXBYE {0}".format(type(SIDE_EFFECTS['Memory']))
        )


class FetchUserData(OAuthCallbackWithRequestToken):

    REQUEST_TOKEN = {
        'oauth_token': 'JL0KLSpLpmWHOMwf',
        'oauth_token_secret': 'SoIvesLtE2KuJrNO',
        'oauth_callback_confirmed': 'true',
    }
    ACCESS_TOKEN = {
        'oauth_token_secret': u'pqtkne3xOh838vtJ',
        'oauth_token': u'pSKTtK9DdUrypvyx',
        'oauth_callback_confirmed': u'false'
    }
    VERIFIER = "28583669"

    def _get_real_session(self, client):
        if 'django.contrib.sessions' in settings.INSTALLED_APPS:
            engine = import_module(settings.SESSION_ENGINE)
            cookie = client.cookies.get(settings.SESSION_COOKIE_NAME, None)
            return engine.SessionStore(cookie and cookie.value or None)

    def test_callback_adds_access_token_to_session(self):
        session = self._get_real_session(self.client)
        session['request_token'] = {
            self.REQUEST_TOKEN['oauth_token']: self.REQUEST_TOKEN['oauth_token_secret']
        }
        session.save()

        with identity_client.tests.use_sso_cassette('fetch_user_data/active_request_token'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': FetchUserData.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': FetchUserData.VERIFIER,
                }
            )

        self.assertEqual(response.status_code, 302)

        self.assertIn('access_token', self.client.session)
        self.assertEqual(
            self.client.session['access_token'],
            {self.ACCESS_TOKEN['oauth_token']: self.ACCESS_TOKEN['oauth_token_secret']}
        )

    def test_user_data_is_added_to_session(self):
        response = full_oauth_dance(self.client)
        self.assertEqual(response.status_code, 302)
        self.assertNotEqual(self.client.session.get('user_data'), None)

    def test_next_url_is_LOGIN_REDIRECT_URL(self):
        response = full_oauth_dance(self.client)
        self.assertEqual(response.status_code, 302)
        self.assertTrue(response['Location'].endswith(settings.LOGIN_REDIRECT_URL))

    def test_authentication_creates_local_user(self):
        Identity.objects.all().delete()
        response = full_oauth_dance(self.client)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(Identity.objects.count(), 1)

    def test_authentication_creates_local_user_accounts(self):
        serviceAccountModel = get_account_module()
        serviceAccountModel.objects.all().delete()
        response = full_oauth_dance(self.client)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(serviceAccountModel.objects.count(), 2)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Removing unique constraint on 'Identity', fields ['email']
        db.delete_unique('identity_client_identity', ['email'])


    def backwards(self, orm):
        
        # Adding unique constraint on 'Identity', fields ['email']
        db.create_unique('identity_client_identity', ['email'])


    models = {
        'identity_client.accountmember': {
            'Meta': {'object_name': 'AccountMember'},
            '_roles': ('django.db.models.fields.CharField', [], {'max_length': '720'}),
            'account': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['identity_client.ServiceAccount']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'identity': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['identity_client.Identity']"})
        },
        'identity_client.identity': {
            'Meta': {'object_name': 'Identity'},
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '50', 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '100', 'null': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '36'})
        },
        'identity_client.serviceaccount': {
            'Meta': {'object_name': 'ServiceAccount'},
            'expiration': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['identity_client.Identity']", 'through': "orm['identity_client.AccountMember']", 'symmetrical': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'plan_slug': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'url': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True'}),
            'uuid': ('django.db.models.fields.CharField', [], {'max_length': '36'})
        }
    }

    complete_apps = ['identity_client']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import datetime


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AccountMember',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('_roles', models.CharField(max_length=720)),
            ],
            options={
                'verbose_name': 'membro da conta do passaporte web',
                'verbose_name_plural': 'membros de contas do passaporte web',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Identity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('first_name', models.CharField(max_length=50, null=True, verbose_name='first name')),
                ('last_name', models.CharField(max_length=100, null=True, verbose_name='last name')),
                ('email', models.EmailField(max_length=75, verbose_name='e-mail address')),
                ('uuid', models.CharField(unique=True, max_length=36, verbose_name='universally unique id')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('last_login', models.DateTimeField(default=datetime.datetime.now, verbose_name='last login')),
            ],
            options={
                'verbose_name': 'usuario do passaporte web',
                'verbose_name_plural': 'usuarios do passaporte web',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ServiceAccount',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=256)),
                ('uuid', models.CharField(max_length=36)),
                ('plan_slug', models.CharField(max_length=50)),
                ('expiration', models.DateTimeField(null=True)),
                ('url', models.CharField(max_length=1024, null=True)),
                ('members', models.ManyToManyField(to='identity_client.Identity', through='identity_client.AccountMember')),
            ],
            options={
                'verbose_name': 'conta do passaporte web',
                'verbose_name_plural': 'contas do passaporte web',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='accountmember',
            name='account',
            field=models.ForeignKey(to='identity_client.ServiceAccount'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='accountmember',
            name='identity',
            field=models.ForeignKey(to='identity_client.Identity'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
try:
    from django.conf.urls.defaults import *
except ImportError:
    from django.conf.urls import *
from django.contrib.auth import views as auth_views

from identity_client.shortcuts import route

urlpatterns = patterns('identity_client.sso.views',
    url(r'^$', 'initiate', name='request_token'),
    url(r'^callback/$', 'fetch_user_data', name='callback'),
    url(r'^iframe/$', 'render_sso_iframe', name='iframe'),
)



# -*- coding: utf-8 -*-
try:
    import mongoengine
except ImportError:
    import sys
    message = u"ERRO: Você precisa instalar o pacote 'mongoengine' para utilizar esta estratégia de persistência\n"
    sys.stdout.write(message.encode('utf-8'))
    sys.exit(1)

from django.conf import settings as django_settings
from django.test import TestCase as SimpleTestCase

from . import models
from . import settings

__all__ = ['models', 'settings', 'TestCase']


mongoengine.connect(
    django_settings.NOSQL_DATABASES['NAME'],
    host=django_settings.NOSQL_DATABASES['HOST']
)


class TestCase(SimpleTestCase):
    """
    TestCase class that clear the collection between the tests
    """
    def __init__(self, methodName='runtest'):
        self.db = mongoengine.connection._get_db()
        super(TestCase, self).__init__(methodName)

    def _post_teardown(self):
        super(TestCase, self)._post_teardown()
        for collection in self.db.collection_names():
            if collection == 'system.indexes':
                continue
            self.db.drop_collection(collection)

    def _fixture_teardown(self, *args, **kwargs):
        pass



# -*- coding: utf-8 -*-
from mock import Mock, patch
from requests_oauthlib.oauth1_session import TokenRequestDenied

from django.utils.importlib import import_module
from django.test import TestCase
from django.http import HttpRequest
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from .mock_helpers import patch_request

import identity_client
from identity_client.sso.client import SSOClient

__all__ = [
    'SSOClientRequestToken',
    'SSOClientAuthorize',
    'SSOClientAccessToken'
]


class SSOClientAuthorize(TestCase):

    AUTHORIZATION_URL = 'https://sandbox.app.passaporteweb.com.br/sso/authorize/?oauth_token=HsLM5nlBdCpARdrt'
    CALLBACK_URL = 'http://127.0.0.1:9000/sso/callback/?oauth_token=HsLM5nlBdCpARdrt&oauth_verifier=56967615'

    def setUp(self):
        self.sso_client = SSOClient()

    def _get_real_session(self, client):
        if 'django.contrib.sessions' in settings.INSTALLED_APPS:
            engine = import_module(settings.SESSION_ENGINE)
            cookie = client.cookies.get(settings.SESSION_COOKIE_NAME, None)
            return engine.SessionStore(cookie and cookie.value or None)

    @patch.object(SSOClient, 'fetch_request_token', Mock(return_value={'oauth_token':'key', 'oauth_token_secret':'secret'}))
    def test_authorize_invokes_fetch_request_token(self):
        self.sso_client.fetch_request_token.reset_mock()

        request = HttpRequest()
        request.session = self._get_real_session(self.client)
        with identity_client.tests.use_sso_cassette('noop'):
            authorization_url = self.sso_client.authorize(request)

        self.assertTrue(self.sso_client.fetch_request_token.called)
        self.assertEquals(self.sso_client.fetch_request_token.call_count, 1)
        self.assertEquals(self.sso_client.fetch_request_token.call_args, ((), {}))

    def test_authorize_stores_request_token_in_session(self):
        request = HttpRequest()
        request.session = self._get_real_session(self.client)
        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            authorization_url = self.sso_client.authorize(request)

        self.assertTrue('request_token' in request.session.keys())
        self.assertEquals(request.session['request_token'], {
            SSOClientRequestToken.REQUEST_TOKEN['oauth_token']: SSOClientRequestToken.REQUEST_TOKEN['oauth_token_secret']
        })

    def test_authorize_stores_next_url_in_session(self):
        request = HttpRequest()
        request.GET = {REDIRECT_FIELD_NAME: '/oauth-protected-view/'}
        request.session = self._get_real_session(self.client)
        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            authorization_url = self.sso_client.authorize(request)

        self.assertTrue(REDIRECT_FIELD_NAME in request.session.keys())
        self.assertEquals(request.session[REDIRECT_FIELD_NAME], '/oauth-protected-view/')

    def test_authorization_url_is_generated_correctly(self):
        request = HttpRequest()
        request.GET = {REDIRECT_FIELD_NAME: '/oauth-protected-view/'}
        request.session = self._get_real_session(self.client)
        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            authorization_url = self.sso_client.authorize(request)

        self.assertEquals(authorization_url, SSOClientAuthorize.AUTHORIZATION_URL)


class SSOClientRequestToken(TestCase):

    REQUEST_TOKEN = {
        'oauth_token': 'HsLM5nlBdCpARdrt',
        'oauth_token_secret': 'OCDrBYT1mY6Y5kV1',
        'oauth_callback_confirmed': 'true',
    }

    def test_fetch_request_token_success(self):
        sso_client = SSOClient()
        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            request_token = sso_client.fetch_request_token()

        self.assertEqual(SSOClientRequestToken.REQUEST_TOKEN, request_token)

    @patch('identity_client.sso.client.settings')
    def test_token_request_is_denied_when_app_credentials_are_invalid(self, settings_mock):
        settings_mock.PASSAPORTE_WEB = {
            'CONSUMER_TOKEN': 'not a valid token',
            'CONSUMER_SECRET': 'not a valid secret',
        }
        sso_client = SSOClient()
        with identity_client.tests.use_sso_cassette('fetch_request_token/invalid_credentials'):
            self.assertRaises(
                TokenRequestDenied, sso_client.fetch_request_token
            )

    def test_malformed_response_generates_ValueError(self):
        sso_client = SSOClient()
        with identity_client.tests.use_sso_cassette('fetch_request_token/malformed_response'):
            self.assertRaises(ValueError, sso_client.fetch_request_token)

    @patch_request(Mock(side_effect=MemoryError))
    def test_abnormal_exceptions_are_not_handled(self):
        sso_client = SSOClient()
        self.assertRaises(MemoryError, sso_client.fetch_request_token)


class SSOClientAccessToken(TestCase):

    VERIFIER = '56967615'
    ACCESS_TOKEN = {
        'oauth_token': 'wT06MCzJYkEVmcLE',
        'oauth_token_secret': 'Xq2U3m3oaKkVfQZ6',
        'oauth_callback_confirmed': 'false',
    }

    def setUp(self):
        self.sso_client = SSOClient(
            resource_owner_key = SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
            resource_owner_secret = SSOClientRequestToken.REQUEST_TOKEN['oauth_token_secret'],
            verifier = SSOClientAccessToken.VERIFIER
        )

    def test_fetch_access_token_success(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/success'):
            access_token = self.sso_client.fetch_access_token()

        self.assertEqual(SSOClientAccessToken.ACCESS_TOKEN, access_token)

    @patch('identity_client.sso.client.settings')
    def test_access_token_request_is_denied_when_app_credentials_are_invalid(self, settings_mock):
        settings_mock.PASSAPORTE_WEB = {
            'CONSUMER_TOKEN': 'not a valid token',
            'CONSUMER_SECRET': 'not a valid secret',
        }
        sso_client = SSOClient(
            resource_owner_key = SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
            resource_owner_secret = SSOClientRequestToken.REQUEST_TOKEN['oauth_token_secret'],
            verifier = SSOClientAccessToken.VERIFIER
        )
        with identity_client.tests.use_sso_cassette('fetch_access_token/invalid_credentials'):
            self.assertRaises(
                TokenRequestDenied, sso_client.fetch_access_token
            )

    def test_access_token_request_is_denied_when_owner_key_is_invalid(self):
        sso_client = SSOClient(
            resource_owner_key = 'invalid owner key',
            resource_owner_secret = SSOClientRequestToken.REQUEST_TOKEN['oauth_token_secret'],
            verifier = SSOClientAccessToken.VERIFIER
        )
        with identity_client.tests.use_sso_cassette('fetch_access_token/invalid_owner_key'):
            self.assertRaises(
                TokenRequestDenied, sso_client.fetch_access_token
            )

    def test_access_token_request_is_denied_when_owner_secret_is_invalid(self):
        sso_client = SSOClient(
            resource_owner_key = SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
            resource_owner_secret = 'invalid owner secret',
            verifier = SSOClientAccessToken.VERIFIER
        )
        with identity_client.tests.use_sso_cassette('fetch_access_token/invalid_owner_secret'):
            self.assertRaises(
                TokenRequestDenied, sso_client.fetch_access_token
            )

    def test_access_token_request_is_denied_when_verifier_is_invalid(self):
        sso_client = SSOClient(
            resource_owner_key = SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
            resource_owner_secret = SSOClientRequestToken.REQUEST_TOKEN['oauth_token_secret'],
            verifier = 'invalid verifier'
        )
        with identity_client.tests.use_sso_cassette('fetch_access_token/invalid_verifier'):
            self.assertRaises(
                TokenRequestDenied, sso_client.fetch_access_token
            )

    def test_malformed_response_generates_ValueError(self):
        sso_client = SSOClient()
        with identity_client.tests.use_sso_cassette('fetch_access_token/malformed_response'):
            self.assertRaises(ValueError, sso_client.fetch_access_token)

    @patch_request(Mock(side_effect=MemoryError))
    def test_abnormal_exceptions_are_not_handled(self):
        self.assertRaises(MemoryError, self.sso_client.fetch_access_token)



# -*- coding: utf-8 -*-
try:
    from django.conf.urls.defaults import *
except ImportError:
    from django.conf.urls import *

from django.contrib.auth.views import logout

urlpatterns = patterns('identity_client.views',
    url(r'^sso/', include('identity_client.sso.urls', namespace='sso_consumer')),
    url(r'^logout/$',
        logout,
        {'template_name': 'logout.html'},
        name='auth_logout'
    ),
)



# -*- encoding: utf-8 -*-
from django.http import Http404
try:
    from django.conf.urls.defaults import url
except ImportError:
    from django.conf.urls import url


def route_by_method(request, *args, **kwargs):
    valid_methods = ('GET', 'POST', 'PUT', 'DELETE')

    method_views = dict(
        (k, v) for k, v in kwargs.items() if k in valid_methods
    )

    try:
        requested_view = method_views[request.method]
        return requested_view(request, *args, **kwargs)
    except KeyError:
        raise Http404


def route(regex, GET=None, POST=None, PUT=None, DELETE=None, kwargs=None, name=None, prefix=''):
    kwargs = kwargs or {}

    kwargs.update({
        'GET': GET,
        'POST': POST,
        'PUT': PUT,
        'DELETE': DELETE
    })

    return url(regex, route_by_method, kwargs, name, prefix)



# -*- coding: utf-8 -*-
from identity_client.urls import *
from django.contrib.auth import views as auth_views

from identity_client.shortcuts import route
from identity_client.views import login, show_login
from identity_client.views import new_identity, register
from identity_client.forms import IdentityAuthenticationForm, RegistrationForm

urlpatterns += patterns('identity_client.views',
    route(r'^registration/',
        GET=new_identity,
        POST=register,
        kwargs={'registration_form': RegistrationForm},
        name='registration_register'
    ),
    route(r'^login/$',
        GET=show_login,
        POST=login,
        kwargs={'authentication_form': IdentityAuthenticationForm},
        name='auth_login'
    ),
     url(r'^accounts/$',
        'list_accounts',
        name='list_accounts'
    ),
     url(r'^alternate_accounts/$',
        'list_accounts',
        name='alternate_accounts'
    ),
)



#coding: utf-8
from django.test.simple import *

from mongoengine import connection

__all__ = ['MongoEngineTestCase']

class MongoEngineTestCase(TestCase):
    """
    TestCase class that clear the collection between the tests
    """
    def __init__(self, methodName='runtest'):
        self.db = connection._get_db()
        super(MongoEngineTestCase, self).__init__(methodName)

    def _post_teardown(self):
        super(MongoEngineTestCase, self)._post_teardown()
        for collection in self.db.collection_names():
            if collection == 'system.indexes':
                continue
            self.db.drop_collection(collection)



# -*- coding: utf-8 -*-
from .helpers import use_cassette, use_sso_cassette

from .test_backend import *
from .test_forms import *
from .test_models import *
from .test_client_views import *
from .test_p3p_middleware import *
from .test_client_api_methods import *
from .test_sso_client import *
from .test_decorators import *
from .test_sso_views import *



# -*- coding: utf-8 -*-
import logging
import six
from requests.exceptions import ReadTimeout, RequestException
from requests_oauthlib.oauth1_session import TokenRequestDenied

from django.contrib.auth.decorators import user_passes_test
from django.http import HttpResponseServerError, HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.conf import settings

from identity_client.sso.client import SSOClient


__all__ = ['sso_login_required', 'oauth_callback']


def sso_login_required(view):

    def decorated(request, *args, **kwargs):
        url = reverse('sso_consumer:request_token')

        actual_decorator = user_passes_test(
            lambda user: user.is_authenticated(),
            login_url=url,
        )

        wrapped_view = actual_decorator(view)

        return wrapped_view(request, *args, **kwargs)

    return decorated


def oauth_callback(view):

    def decorated(request, *args, **kwargs):
        if 'request_token' not in request.session:
            # O fluxo de autenticação ainda não foi iniciado

            # Salvar a url do callback na sessão, ignorando sua querystring
            request.session['callback_url'] = u'{0}{1}'.format(
                settings.APPLICATION_HOST, request.path
            )
            request.session.save()

            return HttpResponseRedirect(reverse('sso_consumer:request_token'))

        else:
            # O fluxo de autenticação já foi iniciado
            if  getattr(request, 'access_token', None) is None:
                # Obter access token
                oauth_token = request.GET.get('oauth_token')
                oauth_verifier = request.GET.get('oauth_verifier')

                request_token = request.session['request_token']
                oauth_secret = request_token[oauth_token]
                try:
                    # Adicionar access token à sessão
                    access_token = SSOClient(
                        resource_owner_key=oauth_token,
                        resource_owner_secret=oauth_secret,
                        verifier=oauth_verifier
                    ).fetch_access_token()
                    request.session['access_token'] = {
                        access_token['oauth_token']: access_token['oauth_token_secret']
                    }
                    request.session.save()

                    # E à request
                    request.access_token = {
                        'resource_owner_key': access_token['oauth_token'],
                        'resource_owner_secret': access_token['oauth_token_secret']
                    }

                except ReadTimeout as e:
                    message = e.args[0].args[0]
                    logging.error(message)
                    return HttpResponseServerError(content=message, status=503)

                except RequestException as e:
                    message = u"Could not fetch access token. {0} ({1})".format(e, e.response.text)
                    logging.error(message)
                    return HttpResponseServerError(content=message, status=503)

                except TokenRequestDenied as e:
                    message = u';'.join(e.args)
                    logging.error(message)
                    return HttpResponseServerError(content=message, status=503)

                except ValueError as e:
                    message = u"Could not fetch access token. {0}".format(e)
                    logging.error(message)
                    return HttpResponseServerError(content=message)

                except Exception as e:
                    message = u"Could not fetch access token. Unknown error ({0})".format(e)
                    logging.error(message)
                    return HttpResponseServerError(content=message)

            return view(request, *args, **kwargs)

    return decorated



# -*- coding: utf-8 -*-
import six
from datetime import datetime, timedelta
import json
import requests
import futures

from mock import patch

from django.conf import settings

import identity_client
from identity_client import PERSISTENCE_MODULE
from identity_client.tests.helpers import MyfcIDTestCase as TestCase
from identity_client.models import Identity, ServiceAccount

__all__ = [
    'TestIdentityModel', 'TestServiceAccountModel'
]

mocked_accounts_json = '''[
    {
        "service_data": {
            "name": "John App", "slug": "johnapp"
        },
        "account_data": {
            "name": "Pessoal",
            "uuid": "e823f8e7-962c-414f-b63f-6cf439686159"
        },
        "plan_slug": "plus",
        "roles": ["owner"],
        "membership_details_url": "%s/organizations/api/accounts/e823f8e7-962c-414f-b63f-6cf439686159/members/1e73dad8-fefe-4b3a-a1a1-7149633748f2/",
        "url": "%s/organizations/api/accounts/e823f8e7-962c-414f-b63f-6cf439686159/",
        "expiration": "%s",
        "external_id": null
    },
    {
        "service_data": {
            "name": "John App", "slug": "johnapp"
        },
        "account_data": {
            "name": "Myfreecomm",
            "uuid": "b39bad59-94af-4880-995a-04967b454c7a"
        },
        "plan_slug": "seller",
        "roles": ["owner"],
        "membership_details_url": "%s/organizations/api/accounts/b39bad59-94af-4880-995a-04967b454c7a/members/1e73dad8-fefe-4b3a-a1a1-7149633748f2/",
        "url": "%s/organizations/api/accounts/b39bad59-94af-4880-995a-04967b454c7a/",
        "expiration": "%s",
        "external_id": null
    },
    {
        "name": "Quebrando os testes",
        "uuid": "dab40435-45bc-45e1-b27e-f0a7c95739df"
    }
]''' % (
    settings.PASSAPORTE_WEB['HOST'],
    settings.PASSAPORTE_WEB['HOST'],
    (datetime.today() + timedelta(days=1)).strftime('%Y-%m-%d'),
    settings.PASSAPORTE_WEB['HOST'],
    settings.PASSAPORTE_WEB['HOST'],
    (datetime.today() + timedelta(days=30)).strftime('%Y-%m-%d')
)

mocked_accounts_list = json.loads(mocked_accounts_json)


class TestIdentityModel(TestCase):


    def setUp(self):
        self.email=u'teste@email.com'
        self.identity = Identity(
            first_name='Teste',
            last_name='Sobrenome',
            email=self.email,
            uuid='16fd2706-8baf-433b-82eb-8c7fada847da',
        )


    def test_unicode(self):
        # An Identity should always be authenticated
        self.assertEqual(six.text_type(self.identity), self.email)


    def test_always_authenticated(self):
        # An Identity should always be authenticated
        self.assertTrue(self.identity.is_authenticated())


    def test_never_anonymous(self):
        # An Identity should never be anonymous
        self.assertEqual(self.identity.is_anonymous(), False)


    def test_set_password_not_implemented(self):
        # This method should not be implemented
        self.assertRaises(
            NotImplementedError,
            self.identity.set_password,
            's3nH4'
        )


    def test_check_password_not_implemented(self):
        # This method should not be implemented
        self.assertRaises(
            NotImplementedError,
            self.identity.check_password,
            's3nH4'
        )


    def test_set_unusable_password_not_implemented(self):
        # This method should not be implemented
        self.assertRaises(
            NotImplementedError,
            self.identity.set_unusable_password,
        )



    def test_has_usable_password_always_false(self):
        # An Identity should never have a usable password
        self.assertEqual(self.identity.has_usable_password(), False)


class TestServiceAccountModel(TestCase):

    def setUp(self):
        self.email = u'teste@email.com'
        self.identity = Identity.objects.create(
            first_name = 'Teste',
            last_name = 'Sobrenome',
            email = self.email,
            uuid = '16fd2706-8baf-433b-82eb-8c7fada847da',
        )

        self.account_name = 'Test Account'
        self.account = ServiceAccount.objects.create(
            name = self.account_name,
            uuid = '16fd2706-8baf-433b-82eb-8c7fada847da',
            plan_slug = 'platinum-test-plan',
        )


    def test_unicode(self):
        # An Identity should always be authenticated
        self.assertEqual(six.text_type(self.account), self.account_name)


    def test_account_without_expiration_is_active(self):
        self.assertEqual(self.account.expiration, None)
        self.assertTrue(self.account.is_active)


    def test_account_with_expiration_in_the_future_is_active(self):
        self.account.expiration = datetime.today() + timedelta(days=2)
        self.assertTrue(self.account.is_active)


    def test_account_with_expiration_in_the_past_is_not_active(self):
        self.account.expiration = datetime.today() - timedelta(days=2)
        self.assertFalse(self.account.is_active)


    def test_default_members_count_is_zero(self):
        self.assertEqual(self.account.members_count, 0)


    def test_add_member_to_empty_account(self):
        self.account.add_member(self.identity, roles=['user'])
        self.assertEqual(self.account.members_count, 1)


    def test_add_member_who_is_already_a_member_does_not_change_member_count(self):
        self.account.add_member(self.identity, roles=['user'])
        self.account.add_member(self.identity, roles=['user', 'admin'])
        self.assertEqual(self.account.members_count, 1)


    def test_get_member_returns_accountmember(self):
        self.account.add_member(self.identity, roles=['user'])
        member = self.account.get_member(self.identity)
        self.assertEqual(member.identity, self.identity)
        self.assertEqual(member.roles, ['user'])


    def test_get_non_existing_member_returns_none(self):
        member = self.account.get_member(self.identity)
        self.assertEqual(member, None)


    def test_get_member_roles_is_a_list_of_strings(self):
        self.account.add_member(self.identity, roles=['user', 'admin'])
        member = self.account.get_member(self.identity)
        self.assertEqual(sorted(member.roles), ['admin', 'user'])


    def test_add_member_who_is_already_a_member_overwrites_roles(self):
        self.account.add_member(self.identity, roles=['user'])
        self.account.add_member(self.identity, roles=['user', 'admin'])
        member = self.account.get_member(self.identity)
        self.assertEqual(sorted(member.roles), ['admin', 'user'])


    def test_member_may_have_no_roles(self):
        self.account.add_member(self.identity, roles=[])
        member = self.account.get_member(self.identity)
        self.assertEqual(member.roles, [])


    def test_remove_member_returns_account(self):
        self.account.add_member(self.identity, roles=['user'])
        account = self.account.remove_member(self.identity)
        self.assertEqual(account, self.account)
        self.assertEqual(account.members_count, 0)


    def test_remove_non_existing_member_fails_silently(self):
        account = self.account.remove_member(self.identity)
        self.assertEqual(account, self.account)
        self.assertEqual(account.members_count, 0)


    def test_account_without_expiration_is_active(self):
        self.assertEqual(self.account.expiration, None)
        active_accounts = ServiceAccount.active().all()
        self.assertEqual(len(active_accounts), 1)
        self.assertEqual(active_accounts[0], self.account)


    def test_account_with_expiration_in_the_future_is_active(self):
        self.account.expiration = datetime.today() + timedelta(days=2)
        self.account.save()
        active_accounts = ServiceAccount.active().all()
        self.assertEqual(len(active_accounts), 1)
        self.assertEqual(active_accounts[0], self.account)


    def test_account_with_expiration_in_the_past_is_not_active(self):
        self.account.expiration = datetime.today() - timedelta(days=2)
        self.account.save()
        active_accounts = ServiceAccount.active().all()
        self.assertEqual(len(active_accounts), 0)


    def test_accounts_for_identity(self):
        self.account.add_member(self.identity, roles=['user'])
        active_accounts = ServiceAccount.for_identity(self.identity)
        self.assertEqual(len(active_accounts), 1)
        self.assertEqual(active_accounts[0], self.account)


    def test_accounts_for_identity_without_accounts_associated(self):
        active_accounts = ServiceAccount.for_identity(self.identity)
        self.assertEqual(len(active_accounts), 0)


    def test_accounts_for_identity_ignores_expired_accounts_by_default(self):
        self.account.expiration = datetime.today() - timedelta(days=2)
        self.account.add_member(self.identity, roles=[])

        active_accounts = ServiceAccount.for_identity(self.identity)
        self.assertEqual(len(active_accounts), 0)


    def test_accounts_for_identity_may_include_expired_accounts(self):
        self.account.expiration = datetime.today() - timedelta(days=2)
        self.account.add_member(self.identity, roles=[])

        active_accounts = ServiceAccount.for_identity(self.identity, include_expired=True)
        self.assertEqual(len(active_accounts), 1)
        self.assertEqual(active_accounts[0], self.account)


    @patch.object(PERSISTENCE_MODULE.models.APIClient, 'fetch_user_accounts')
    def test_pull_remote_accounts_changes_response_format(self, mocked_accounts):
        mocked_accounts.return_value = 200, mocked_accounts_list, []
        accounts = ServiceAccount.pull_remote_accounts(self.identity)

        expected = [
            {
                'plan_slug': mocked_accounts_list[0]['plan_slug'],
                'uuid': mocked_accounts_list[0]['account_data']['uuid'],
                'roles': mocked_accounts_list[0]['roles'],
                'url': mocked_accounts_list[0]['url'],
                'name': mocked_accounts_list[0]['account_data']['name'],
                'expiration': mocked_accounts_list[0]['expiration'],
            },
            {
                'plan_slug': mocked_accounts_list[1]['plan_slug'],
                'uuid': mocked_accounts_list[1]['account_data']['uuid'],
                'roles': mocked_accounts_list[1]['roles'],
                'url': mocked_accounts_list[1]['url'],
                'name': mocked_accounts_list[1]['account_data']['name'],
                'expiration': mocked_accounts_list[1]['expiration'],
            },
            {
                'plan_slug': 'UNKNOWN',
                'uuid': mocked_accounts_list[2]['uuid'],
                'roles': [],
                'url': None,
                'name': mocked_accounts_list[2]['name'],
                'expiration': None,
            },
        ]

        self.assertEqual(list(accounts), expected)


    @patch.object(PERSISTENCE_MODULE.models.APIClient, 'fetch_user_accounts')
    def test_update_user_accounts_creates_local_accounts(self, mocked_accounts):
        mocked_accounts.return_value = 200, mocked_accounts_list, []

        self.assertEqual(ServiceAccount.for_identity(self.identity).count(), 0)

        accounts = ServiceAccount.pull_remote_accounts(self.identity)
        ServiceAccount.update_user_accounts(self.identity, accounts)

        self.assertEqual(ServiceAccount.for_identity(self.identity).count(), 3)


    @patch.object(PERSISTENCE_MODULE.models.APIClient, 'fetch_user_accounts')
    def test_remove_stale_accounts(self, mocked_accounts):
        mocked_accounts.return_value = 200, mocked_accounts_list, []

        self.account.add_member(self.identity, roles=['user'])
        self.assertEqual(ServiceAccount.for_identity(self.identity).count(), 1)

        accounts = ServiceAccount.pull_remote_accounts(self.identity)
        ServiceAccount.remove_stale_accounts(self.identity, accounts)

        self.assertEqual(ServiceAccount.for_identity(self.identity).count(), 0)


    @patch.object(PERSISTENCE_MODULE.models.APIClient, 'fetch_user_accounts')
    def test_refresh_user_accounts(self, mocked_accounts):
        """
        This method reads the remote user's accounts,
        updates them locally and purges the accounts which do not exist anymore.
        """
        mocked_accounts.return_value = 200, mocked_accounts_list, []

        self.account.add_member(self.identity, roles=['user'])
        self.assertEqual(ServiceAccount.for_identity(self.identity).count(), 1)

        ServiceAccount.refresh_accounts(self.identity)

        self.assertEqual(ServiceAccount.for_identity(self.identity).count(), 3)
        self.assertFalse(self.account in ServiceAccount.for_identity(self.identity))

    def test_send_notification(self):
        self.account.url = "https://sandbox.app.passaporteweb.com.br/organizations/api/accounts/e5ab6f2f-a4eb-431b-8c12-9411fd8a872d/"
        body = "Test notification for the account"
        with identity_client.tests.use_cassette('service_account/send_notification'):
            future = self.account.send_notification(body)

        self.assertTrue(isinstance(future, futures.Future))
        self.assertEqual(future.result().body, body)

    def test_error_sending_notification(self):
        self.account.url = "https://sandbox.app.passaporteweb.com.br/organizations/api/accounts/e5ab6f2f-a4eb-431b-8c12-9411fd8a872d/"
        body = "Test notification for the account"
        with identity_client.tests.use_cassette('service_account/403_sending_notification'):
            # The call won't fail
            future = self.account.send_notification(body)

        # Buy the result will contain the raised exception
        self.assertTrue(isinstance(future, futures.Future))
        self.assertRaises(requests.HTTPError, future.result)



# -*- coding: utf-8 -*-
from mock import Mock, patch

__all__ = ['patch_request']

def patch_request(new):
    return patch('requests.packages.urllib3.connectionpool.HTTPConnectionPool._make_request', new)



# -*- coding: utf-8 -*-
import logging
from mock import Mock, patch
from requests.packages.urllib3.exceptions import ReadTimeoutError

from django.utils.importlib import import_module
from django.test import TestCase
from django.core.urlresolvers import reverse
from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME

import identity_client

from identity_client.utils import reverse_with_host
from identity_client.tests.test_sso_client import (
    SSOClientRequestToken, SSOClientAccessToken, SSOClientAuthorize
)
from .helpers import full_oauth_dance
from .mock_helpers import patch_request

__all__ = [
    'OAuthCallbackWithoutRequestToken',
    'OAuthCallbackWithRequestToken',
    'SSOLoginRequired',
]

from warnings import warn;
warn('decorators.requires_plan não está testado')

SIDE_EFFECTS = {
    'Timeout': ReadTimeoutError('connection', 'url', "Read timed out. (read timeout=30)"),
    'Memory': MemoryError('KTHXBYE'),
}


class OAuthCallbackWithoutRequestToken(TestCase):

    def test_redirects_to_sso_on_missing_request_token(self):
        self.assertNotIn('request_token', self.client.session)

        response = self.client.get(
            reverse('sso_consumer:callback'), {
                'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                'oauth_verifier': SSOClientAccessToken.VERIFIER
            }
        )

        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], reverse_with_host('sso_consumer:request_token'))

    def test_adds_callback_url_to_session(self):
        self.assertNotIn('request_token', self.client.session)

        response = self.client.get(reverse('sso_consumer:callback'))

        self.assertEqual(response.status_code, 302)
        self.assertIn('callback_url', self.client.session)
        self.assertEqual(self.client.session['callback_url'], reverse_with_host('sso_consumer:callback'))

    def test_strips_querystring_before_adding_callback_url_to_session(self):
        self.assertNotIn('request_token', self.client.session)

        response = self.client.get(
            reverse('sso_consumer:callback'), {'key': 'value', 'key2': 'value2'}
        )

        self.assertEqual(response.status_code, 302)
        self.assertIn('callback_url', self.client.session)
        self.assertEqual(self.client.session['callback_url'], reverse_with_host('sso_consumer:callback'))


class OAuthCallbackWithRequestToken(TestCase):

    def setUp(self):
        with identity_client.tests.use_sso_cassette('fetch_request_token/success'):
            self.client.get(reverse('sso_consumer:request_token'), {})

        self.assertIn('request_token', self.client.session)

    def test_callback_fails_if_service_credentials_are_invalid(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/invalid_credentials'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b"Token request failed with code 401, response was 'Unauthorized consumer with key 'not a valid token''."
        self.assertEqual(response.content, expected)
        self.assertNotIn('access_token', self.client.session)

    def test_callback_fails_if_connection_to_provider_fails(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/500'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b"Token request failed with code 500, response was 'Internal Server Error'."
        self.assertEqual(response.content, expected)
        self.assertNotIn('access_token', self.client.session)

    def test_callback_fails_if_connection_to_provider_times_out(self):
        with patch_request(Mock(side_effect=SIDE_EFFECTS['Timeout'])):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b'connection: Read timed out. (read timeout=30)'
        self.assertEqual(response.content, expected)
        self.assertNotIn('access_token', self.client.session)

    def test_callback_fails_if_provider_is_not_available(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/503'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b"Token request failed with code 503, response was 'Service Unavailable'."
        self.assertEqual(response.content, expected)
        self.assertNotIn('access_token', self.client.session)

    def test_callback_fails_if_request_has_an_error(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/418'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b"Token request failed with code 418, response was 'I'm a Teapot'."
        self.assertEqual(response.content, expected)
        self.assertNotIn('access_token', self.client.session)

    def test_callback_fails_gracefully_when_response_is_malformed(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/malformed_response'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 500)
        self.assertNotIn('access_token', self.client.session)

        content_start = response.content[:72]
        expected_start = b'Could not fetch access token. Unable to decode token from token response'
        self.assertEqual(content_start, expected_start)

        content_end = response.content[-65:]
        expected_end = b'Please ensure the request/response body is x-www-form-urlencoded.'
        self.assertEqual(content_end, expected_end)

    def test_callback_fails_gracefully(self):
        with patch_request(Mock(side_effect=SIDE_EFFECTS['Memory'])):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 500)
        self.assertEqual(response.content, b"Could not fetch access token. Unknown error (KTHXBYE)")
        self.assertNotIn('access_token', self.client.session)

    def test_callback_failure_when_verifier_is_invalid(self):
        with identity_client.tests.use_sso_cassette('fetch_access_token/invalid_verifier'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': '56967615'
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b"Token request failed with code 401, response was 'invalid oauth_token verifier: 56967615'."
        self.assertEqual(response.content, expected)
        self.assertNotIn('access_token', self.client.session)

    def test_fails_when_access_token_is_expired(self):
        with identity_client.tests.use_sso_cassette('fetch_user_data/expired_access_token'):
            response = self.client.get(
                reverse('sso_consumer:callback'), {
                    'oauth_token': SSOClientRequestToken.REQUEST_TOKEN['oauth_token'],
                    'oauth_verifier': SSOClientAccessToken.VERIFIER
                }
            )

        self.assertEqual(response.status_code, 503)
        expected = b'Error invoking decorated: 401 Client Error: UNAUTHORIZED ({"detail": "You need to login or otherwise authenticate the request."})'
        self.assertEqual(response.content, expected)
        self.assertEqual(
            self.client.session['access_token'],
            {SSOClientAccessToken.ACCESS_TOKEN['oauth_token']: SSOClientAccessToken.ACCESS_TOKEN['oauth_token_secret']}
        )


class SSOLoginRequired(TestCase):

    def test_anonymous_user_is_redirected_to_initiate(self):
        expected_redirect = u'{0}?next={1}'.format(
            reverse_with_host('sso_consumer:request_token'), settings.LOGIN_REDIRECT_URL
        )
        response = self.client.get(settings.LOGIN_REDIRECT_URL, follow=False)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], expected_redirect)

    def test_requested_url_will_be_used_as_next_url(self):
        requested_url = reverse('alternate_accounts')
        expected_redirect = u'{0}?next={1}'.format(
            reverse_with_host('sso_consumer:request_token'), requested_url
        )
        response = self.client.get(requested_url, follow=False)
        self.assertEqual(response.status_code, 302)
        self.assertEqual(response['Location'], expected_redirect)

    def test_authenticated_user_is_allowed_through(self):
        sso_response = full_oauth_dance(self.client)
        with identity_client.tests.use_sso_cassette('fetch_user_data/active_request_token'):
            first_response = self.client.get(sso_response['Location'])
            requested_url = reverse('alternate_accounts')
            second_response = self.client.get(requested_url, follow=False)

        self.assertEqual(second_response.status_code, 200)
